package com.sw.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sw.dao.SysUserDao;
import com.sw.dao.TMaskDao;
import com.sw.dao.TOrderDao;
import com.sw.domain.TMask;
import com.sw.domain.TOrder;
import com.sw.service.TMaskService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;

/**
 * (TMask)表服务实现类
 *
 * @author yegang
 * @since 2022-02-18 10:27:46
 */
@Slf4j
@Service("tMaskService")
public class TMaskServiceImpl extends ServiceImpl<TMaskDao, TMask> implements TMaskService {
    @Resource
    private TMaskDao tMaskDao;
    @Autowired
    private SysUserDao sysUserDao;
    @Autowired
    private TOrderDao tOrderDao;
    @Autowired
    private RedisTemplate redisTemplate;
    @Override
    /*
     * 方法描述:秒杀service
     * @param: Integer mid, Integer uid
     * @return: String
     * @author: yegang
     * @date: 2022/2/18
     */
    @Transactional(rollbackFor = Exception.class)
    public String spike(Integer mid, Integer uid) {
        //1.判断库存
        TMask tMask = tMaskDao.selectById(mid);
        if(tMask.getMaskStock()<5){
            return "库存不够，抢购失败";
        }
        //2.减库存
        boolean b = tMaskDao.decreaseStock(5, tMask.getVersion(), mid);
        if(!b){
//            log.error("减库存成功");
            return "抢购失败,请重试";
        }
        //3.下订单
        TOrder tOrder =new TOrder();
        tOrder.setCreateTime(new Date());
        tOrder.setState("1");
        tOrder.setMid(mid);
        tOrder.setUid(uid);
//        log.error("insert前");
        int insert = tOrderDao.insert(tOrder);
//        log.error("insert后");
        //4.设置成功缓存
        redisTemplate.opsForSet().add("sucess-" + mid,uid);
        return "抢购成功，订单号为"+tOrder.getId();
    }
    @Override
    public boolean decrease(Integer maskId, Integer remain) {
        UpdateWrapper<TMask> update = new UpdateWrapper<TMask>();
        update.eq("id", maskId)
                .set("mask_stock", remain);
        return this.update(update);
    }
}
